#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "teamspeak/public_errors.h"
#include "teamspeak/public_errors_rare.h"
#include "teamspeak/public_definitions.h"
#include "teamspeak/public_rare_definitions.h"
#include "teamspeak/clientlib_publicdefinitions.h"
#include "ts3_functions.h"

#include "plugin.h"
#include "../../tsgo_export.h"
#include "ts3FunctionsWrapper.h"

static struct TS3Functions ts3Functions;

#define _strcpy(dest, destSize, src) { strncpy(dest, src, destSize-1); (dest)[destSize-1] = '\0'; }

static char* pluginID = NULL;

/*********************************** Required functions ************************************/
/*
 * If any of these required functions is not implemented, TS3 will refuse to load the plugin
 */

/* Unique name identifying this plugin */
const char* ts3plugin_name() {
	return GoTs3pluginName();
}

/* Plugin version */
const char* ts3plugin_version() {
    return GoTs3pluginVersion();
}

/* Plugin API version. Must be the same as the clients API major version, else the plugin fails to load. */
int ts3plugin_apiVersion() {
	return GoTs3pluginAPIVersion();
}

/* Plugin author */
const char* ts3plugin_author() {
    return GoTs3pluginAuthor();
}

/* Plugin description */
const char* ts3plugin_description() {
    return GoTs3pluginDescription();
}

/* Set TeamSpeak 3 callback functions */
void ts3plugin_setFunctionPointers(const struct TS3Functions funcs) {
    ts3Functions = funcs;
}

/*
 * Custom code called right after loading the plugin. Returns 0 on success, 1 on failure.
 * If the function returns 1 on failure, the plugin will be unloaded again.
 */
int ts3plugin_init() {

	return GoTs3pluginInit();
    /* 0 = success, 1 = failure, -2 = failure but client will not show a "failed to load" warning */
	/* -2 is a very special case and should only be used if a plugin displays a dialog (e.g. overlay) asking the user to disable
	 * the plugin again, avoiding the show another dialog by the client telling the user the plugin failed to load.
	 * For normal case, if a plugin really failed to load because of an error, the correct return value is 1. */
}

/* Custom code called right before the plugin is unloaded */
void ts3plugin_shutdown() {

	GoTs3pluginShutdown();

	/* Free pluginID if we registered it */
	if(pluginID) {
		free(pluginID);
		pluginID = NULL;
	}
}


/*
	Not so required functions
*/

int ts3plugin_onClientPokeEvent(uint64 serverConnectionHandlerID, anyID fromClientID, const char* pokerName, const char* pokerUniqueIdentity, const char* message, int ffIgnored) {
    return GoTs3onClientPokeEvent(serverConnectionHandlerID,fromClientID,pokerName,pokerUniqueIdentity,message,ffIgnored);
}

/*
	Ts3 Functions

	This wrapper is required beacuse calling function pointers is not supported in cgo.
*/

void Ts3FncPrintMessage(uint64 serverConnectionHandlerID, const char* message, enum PluginMessageTarget messageTarget){
	ts3Functions.printMessage(serverConnectionHandlerID,message,messageTarget);
}
